@@ -4,12 +4,12 @@ import ddt
 import pyparsing as pp
 from iker.common.utils.dtutils import dt_parse_iso
 
-from plexus.common.utils.strutils import BagName, UserName, VehicleName
+from plexus.common.utils.strutils import BagName, UserEmail, UserName, VehicleName
 from plexus.common.utils.strutils import colon_tag_parser, colon_tag_pattern, slash_tag_parser, slash_tag_pattern
 from plexus.common.utils.strutils import dot_case_parser, dot_case_pattern
 from plexus.common.utils.strutils import hex_string_parser, hex_string_pattern
 from plexus.common.utils.strutils import kebab_case_parser, kebab_case_pattern
-from plexus.common.utils.strutils import parse_bag_name, parse_user_name, parse_vehicle_name
+from plexus.common.utils.strutils import parse_bag_name, parse_user_email, parse_user_name, parse_vehicle_name
 from plexus.common.utils.strutils import semver_parser, semver_pattern
 from plexus.common.utils.strutils import snake_case_parser, snake_case_pattern
 from plexus.common.utils.strutils import strict_abspath_parser, strict_abspath_pattern
@@ -568,6 +568,51 @@ class StrUtilsTest(unittest.TestCase):
         with self.assertRaises(pp.ParseException):
             parse_user_name(data)
 
+    data_parse_user_email = [
+        ("dummy.person@some.company", UserEmail(UserName("dummy", "person", 0), "some.company")),
+        ("dummy1.person@another.company", UserEmail(UserName("dummy", "person", 1), "another.company")),
+        ("dummy1.person@yet-another.company", UserEmail(UserName("dummy", "person", 1), "yet-another.company")),
+        ("dummy1.person@yet-yet.another.company", UserEmail(UserName("dummy", "person", 1), "yet-yet.another.company")),
+        ("dummy999999.person@good.company", UserEmail(UserName("dummy", "person", 999999), "good.company")),
+        ("dummy.person@better.or.even-better.company",
+         UserEmail(UserName("dummy", "person", 0), "better.or.even-better.company")),
+    ]
+
+    @ddt.idata(data_parse_user_email)
+    @ddt.unpack
+    def test_parse_user_email(self, data, expect):
+        self.assertEqual(parse_user_email(data), expect)
+        self.assertEqual(data, str(expect))
+
+    data_parse_user_email__bad_cases = [
+        ("dummy",),
+        ("0.dummy",),
+        ("dummy.0",),
+        ("Dummy.Dummy",),
+        ("dummy-dummy.dummy",),
+        ("0dummy.dummy",),
+        ("0dummy0.dummy",),
+        ("dummy01.dummy",),
+        ("dummy@some.company",),
+        ("0.dummy@some.company",),
+        ("dummy.0@some.company",),
+        ("Dummy.Dummy@some.company",),
+        ("dummy-dummy.dummy@some.company",),
+        ("0dummy.dummy@some.company",),
+        ("0dummy0.dummy@some.company",),
+        ("dummy01.dummy@some.company",),
+        ("dummy.person@BAD.COMPANY",),
+        ("dummy1.person@worse.company.",),
+        ("dummy1.person@even.worse-company",),
+        ("dummy1.person#worst.company",),
+    ]
+
+    @ddt.idata(data_parse_user_email__bad_cases)
+    @ddt.unpack
+    def test_parse_user_email__bad_cases(self, data):
+        with self.assertRaises(pp.ParseException):
+            parse_user_email(data)
+
     data_parse_vehicle_name = [
         ("brand_alias",
          VehicleName("brand", "alias", None, None)),
